const get = require('lodash/get')
const pickBy = require('lodash/pickBy')
const omit = require('lodash/omit')
const helpers = require('./authsome-helpers')

async function teamPermissions(user, operation, object, context) {
  const { models } = context
  const permissions = ['handlingEditor', 'author', 'reviewer']
  const teams = await helpers.getTeamsByPermissions(
    user.teams,
    permissions,
    context.models.Team,
  )

  let collectionsPermissions = await Promise.all(
    teams.map(async team => {
      let collection
      if (team.object.type === 'collection') {
        collection = await models.Collection.find(team.object.id)
      } else if (team.object.type === 'fragment') {
        const fragment = await models.Fragment.find(team.object.id)
        collection = await models.Collection.find(fragment.collectionId)
      }
      if (
        collection.status === 'rejected' &&
        team.teamType.permissions === 'reviewer'
      )
        return null
      const collPerm = {
        id: collection.id,
        permission: team.teamType.permissions,
      }
      const objectType = get(object, 'type')
      if (objectType === 'fragment') {
        if (collection.fragments.includes(object.id))
          collPerm.fragmentId = object.id
        else return null
      }

      if (objectType === 'collection')
        if (object.id !== collection.id) return null
      return collPerm
    }),
  )
  collectionsPermissions = collectionsPermissions.filter(cp => cp !== null)
  if (collectionsPermissions.length === 0) return false

  return {
    filter: filterParam => {
      if (!filterParam.length) {
        return helpers.filterObjectData(
          collectionsPermissions,
          filterParam,
          user,
        )
      }

      const collections = filterParam
        .map(coll =>
          helpers.filterObjectData(collectionsPermissions, coll, user),
        )
        .filter(Boolean)
      return collections
    },
  }
}

function unauthenticatedUser(operation, object) {
  // Public/unauthenticated users can GET /collections, filtered by 'published'
  if (operation === 'GET' && object && object.path === '/collections') {
    return {
      filter: collections =>
        collections.filter(collection => collection.published),
    }
  }

  // Public/unauthenticated users can GET /collections/:id/fragments, filtered by 'published'
  if (
    operation === 'GET' &&
    object &&
    object.path === '/collections/:id/fragments'
  ) {
    return {
      filter: fragments => fragments.filter(fragment => fragment.published),
    }
  }

  // and filtered individual collection's properties: id, title, source, content, owners
  if (operation === 'GET' && object && object.type === 'collection') {
    if (object.published) {
      return {
        filter: collection =>
          pickBy(collection, (_, key) =>
            ['id', 'title', 'owners'].includes(key),
          ),
      }
    }
  }

  if (operation === 'GET' && object && object.type === 'fragment') {
    if (object.published) {
      return {
        filter: fragment =>
          pickBy(fragment, (_, key) =>
            ['id', 'title', 'source', 'presentation', 'owners'].includes(key),
          ),
      }
    }
  }

  return false
}

async function authenticatedUser(user, operation, object, context) {
  // Allow the authenticated user to POST a collection (but not with a 'filtered' property)
  if (operation === 'POST' && object.path === '/collections') {
    return {
      filter: collection => omit(collection, 'filtered'),
    }
  }

  if (
    operation === 'POST' &&
    object.path === '/collections/:collectionId/fragments'
  ) {
    return true
  }

  // allow authenticate owners full pass for a collection
  if (get(object, 'type') === 'collection') {
    if (operation === 'PATCH') {
      return {
        filter: collection => omit(collection, 'filtered'),
      }
    }
    if (object.owners.includes(user.id)) return true
    const owner = object.owners.find(own => own.id === user.id)
    if (owner !== undefined) return true
  }

  // Allow owners of a collection to GET its teams, e.g.
  // GET /api/collections/1/teams
  if (operation === 'GET' && get(object, 'path') === '/teams') {
    const collectionId = get(object, 'params.collectionId')
    if (collectionId) {
      const collection = await context.models.Collection.find(collectionId)
      if (collection.owners.includes(user.id)) {
        return true
      }
    }
  }

  if (
    operation === 'GET' &&
    get(object, 'type') === 'team' &&
    get(object, 'object.type') === 'collection'
  ) {
    const collection = await context.models.Collection.find(
      get(object, 'object.id'),
    )
    if (collection.owners.includes(user.id)) {
      return true
    }
  }

  // Advanced example
  // Allow authenticated users to create a team based around a collection
  // if they are one of the owners of this collection
  if (['POST', 'PATCH'].includes(operation) && get(object, 'type') === 'team') {
    if (get(object, 'object.type') === 'collection') {
      const collection = await context.models.Collection.find(
        get(object, 'object.id'),
      )
      if (collection.owners.includes(user.id)) {
        return true
      }
    }
  }

  // only allow the HE to create, delete an invitation, or get invitation details
  if (
    ['POST', 'GET', 'DELETE'].includes(operation) &&
    get(object.collection, 'type') === 'collection' &&
    object.path.includes('invitations')
  ) {
    const collection = await context.models.Collection.find(
      get(object.collection, 'id'),
    )
    const handlingEditor = get(collection, 'handlingEditor')
    if (!handlingEditor) return false
    if (handlingEditor.id === user.id) return true
    return false
  }

  // only allow a reviewer and an HE to submit and to modify a recommendation
  if (
    ['POST', 'PATCH'].includes(operation) &&
    object.path.includes('recommendations')
  ) {
    const authsomeObject = get(object, 'authsomeObject')

    const teams = await helpers.getTeamsByPermissions(
      user.teams,
      ['reviewer', 'handlingEditor'],
      context.models.Team,
    )

    if (teams.length === 0) return false
    const matchingTeam = teams.find(
      team => team.object.id === authsomeObject.id,
    )

    if (matchingTeam) return true
    return false
  }

  if (user.teams.length !== 0 && ['GET'].includes(operation)) {
    const permissions = await teamPermissions(user, operation, object, context)

    if (permissions) {
      return permissions
    }

    return false
  }

  if (get(object, 'type') === 'fragment') {
    const fragment = object

    if (fragment.owners.includes(user.id)) {
      return true
    }
  }

  // A user can GET, DELETE and PATCH itself
  if (get(object, 'type') === 'user' && get(object, 'id') === user.id) {
    if (['GET', 'DELETE', 'PATCH'].includes(operation)) {
      return true
    }
  }
  // If no individual permissions exist (above), fallback to unauthenticated
  // user's permission
  return unauthenticatedUser(operation, object)
}

const authsomeMode = async (userId, operation, object, context) => {
  if (!userId) {
    return unauthenticatedUser(operation, object)
  }

  // It's up to us to retrieve the relevant models for our
  // authorization/authsome mode, e.g.
  const user = await context.models.User.find(userId)

  // Admins and editor in chiefs can do anything
  if (user && (user.admin || user.editorInChief)) return true

  if (user) {
    return authenticatedUser(user, operation, object, context)
  }

  return false
}

module.exports = authsomeMode
