const config = require('config')
const { get, pickBy, omit } = require('lodash')

const statuses = config.get('statuses')
const helpers = require('./authsome-helpers')

function unauthenticatedUser(operation, object) {
  // Public/unauthenticated users can GET /collections, filtered by 'published'
  if (operation === 'GET' && object && object.path === '/collections') {
    return {
      filter: collections =>
        collections.filter(collection => collection.published),
    }
  }

  // Public/unauthenticated users can GET /collections/:id/fragments, filtered by 'published'
  if (
    operation === 'GET' &&
    object &&
    object.path === '/collections/:id/fragments'
  ) {
    return {
      filter: fragments => fragments.filter(fragment => fragment.published),
    }
  }

  // and filtered individual collection's properties: id, title, source, content, owners
  if (operation === 'GET' && object && object.type === 'collection') {
    if (object.published) {
      return {
        filter: collection =>
          pickBy(collection, (_, key) =>
            ['id', 'title', 'owners'].includes(key),
          ),
      }
    }
  }

  if (operation === 'GET' && object && object.type === 'fragment') {
    if (object.published) {
      return {
        filter: fragment =>
          pickBy(fragment, (_, key) =>
            ['id', 'title', 'source', 'presentation', 'owners'].includes(key),
          ),
      }
    }
  }

  return false
}

const publicStatusesPermissions = ['author', 'reviewer']
const createPaths = ['/collections', '/collections/:collectionId/fragments']

async function authenticatedUser(user, operation, object, context) {
  if (operation === 'GET') {
    if (get(object, 'path') === '/collections') {
      return {
        filter: async collections => {
          const userPermissions = await helpers.getUserPermissions({
            user,
            Team: context.models.Team,
          })
          return collections.filter(collection => {
            if (collection.owners.includes(user.id)) {
              return true
            }
            const collectionPermission = userPermissions.find(
              p => p.objectId === collection.id,
            )
            if (collectionPermission) {
              return true
            }

            const fragmentPermission = userPermissions.find(p =>
              collection.fragments.includes(p.objectId),
            )
            if (fragmentPermission) {
              return true
            }
            return false
          })
        },
      }
    }

    if (object === '/users') {
      return true
    }

    if (get(object, 'type') === 'collection') {
      if (helpers.isOwner({ user, object })) {
        return true
      }
      return {
        filter: async collection => {
          const status = get(collection, 'status') || 'draft'
          const userPermissions = await helpers.getUserPermissions({
            user,
            Team: context.models.Team,
          })
          if (collection.owners.map(o => o.id).includes(user.id)) {
            return collection
          }

          const collectionPermission = userPermissions.find(
            p => p.objectId === collection.id,
          )
          if (
            publicStatusesPermissions.includes(
              get(collectionPermission, 'role'),
            )
          ) {
            collection.visibleStatus = statuses[status].public
          }
          return collection
        },
      }
    }

    if (get(object, 'type') === 'fragment') {
      if (helpers.isOwner({ user, object })) {
        return true
      }

      const userPermissions = await helpers.getUserPermissions({
        user,
        Team: context.models.Team,
      })

      const permission = userPermissions.find(
        p => p.objectId === object.id || p.objectId === object.collectionId,
      )

      if (!permission) return false

      return {
        filter: fragment => {
          // handle other roles
          if (permission.role === 'reviewer') {
            fragment.files = omit(fragment.files, ['coverLetter'])
            fragment.authors = fragment.authors.map(a => omit(a, ['email']))
          }
          return fragment
        },
      }
    }

    // allow HE to get reviewer invitations
    if (get(object, 'fragment.type') === 'fragment') {
      const collectionId = get(object, 'fragment.collectionId')
      const collection = await context.models.Collection.find(collectionId)

      if (get(collection, 'handlingEditor.id') === user.id) {
        return true
      }
    }

    if (get(object, 'type') === 'user') {
      return true
    }
  }

  if (operation === 'POST') {
    // allow everytone to create manuscripts and versions
    if (createPaths.includes(object.path)) {
      return true
    }

    // allow HE to invite
    if (
      get(object, 'path') ===
      '/api/collections/:collectionId/fragments/:fragmentId/invitations'
    ) {
      return helpers.isHandlingEditor({ user, object })
    }

    // allow HE or assigned reviewers to recommend
    if (
      get(object, 'path') ===
      '/api/collections/:collectionId/fragments/:fragmentId/recommendations'
    ) {
      return helpers.hasPermissionForObject({
        user,
        object,
        Team: context.models.Team,
        roles: ['reviewer', 'handlingEditor'],
      })
    }
  }

  if (operation === 'PATCH') {
    if (get(object, 'type') === 'collection') {
      return helpers.isOwner({ user, object })
    }

    if (get(object, 'type') === 'fragment') {
      return helpers.isOwner({ user, object })
    }

    // allow reviewer to patch his recommendation
    if (
      get(object, 'path') ===
      '/api/collections/:collectionId/fragments/:fragmentId/recommendations/:recommendationId'
    ) {
      return helpers.hasPermissionForObject({
        user,
        object,
        Team: context.models.Team,
        roles: ['reviewer'],
      })
    }

    if (get(object, 'type') === 'user' && get(object, 'id') === user.id) {
      return true
    }
  }

  if (operation === 'DELETE') {
    if (
      get(object, 'path') ===
      '/api/collections/:collectionId/fragments/:fragmentId/invitations/:invitationId'
    ) {
      return helpers.isHandlingEditor({ user, object })
    }
  }

  // If no individual permissions exist (above), fallback to unauthenticated
  // user's permission
  return unauthenticatedUser(operation, object)
}

const authsomeMode = async (userId, operation, object, context) => {
  if (!userId) {
    return unauthenticatedUser(operation, object)
  }

  // It's up to us to retrieve the relevant models for our
  // authorization/authsome mode, e.g.
  const user = await context.models.User.find(userId)

  // Admins and editor in chiefs can do anything
  if (user && (user.admin || user.editorInChief)) return true

  if (user) {
    return authenticatedUser(user, operation, object, context)
  }

  return false
}

module.exports = authsomeMode
