import PropTypes from 'prop-types'
import { connect } from 'react-redux'
import { actions } from 'pubsweet-client'
import { debounce, pick, get, isEqual } from 'lodash'
import { compose, getContext, withProps, setDisplayName } from 'recompose'
import { reduxForm, formValueSelector, SubmissionError } from 'redux-form'

import WizardStep from './WizardStep'
import { autosaveRequest } from '../redux/autosave'
import { submitRevision, isRevisionFlow } from '../redux/conversion'

const wizardSelector = formValueSelector('wizard')

const onChange = (
  values,
  dispatch,
  { project, version, wizard: { formSectionKeys }, setLoader },
) => {
  const prev = pick(version, formSectionKeys)
  const newValues = pick(values, formSectionKeys)
  // TODO: fix this if it sucks down the road
  if (!isEqual(prev, newValues)) {
    dispatch(autosaveRequest())
    dispatch(
      actions.updateFragment(project, {
        id: version.id,
        ...newValues,
      }),
    )
  }
}

const submitManuscript = ({
  values,
  version,
  project,
  history,
  dispatch,
  redirectPath = '/',
}) => {
  dispatch(
    actions.updateFragment(project, {
      id: version.id,
      submitted: new Date(),
      ...values,
    }),
  )
    .then(() =>
      dispatch(
        actions.updateCollection({
          id: project.id,
          status: 'submitted',
        }),
      ),
    )
    .then(() => {
      history.push(redirectPath, {
        project: project.id,
        customId: project.customId,
        version: version.id,
      })
    })
    .catch(error => {
      if (error.validationErrors) {
        throw new SubmissionError()
      }
    })
}

const submitFragmentRevision = ({
  version,
  project,
  history,
  redirectPath = '/',
}) => {
  submitRevision(project.id, version.id)
    .then(() => {
      history.push(redirectPath, {
        project: project.id,
        customId: project.customId,
        version: version.id,
      })
    })
    .catch(error => {
      if (error.validationErrors) {
        throw new SubmissionError()
      }
    })
}

const onSubmit = (
  values,
  dispatch,
  {
    isFinal,
    history,
    project,
    version,
    nextStep,
    confirmation,
    isRevisionFlow,
    toggleConfirmation,
    wizard: { confirmationModal, submissionRedirect, formSectionKeys },
    ...rest
  },
) => {
  if (!isFinal) {
    nextStep()
  } else if (confirmationModal && !confirmation) {
    toggleConfirmation()
  } else if (isRevisionFlow) {
    submitFragmentRevision({
      version,
      project,
      history,
      redirectPath: submissionRedirect,
    })
  } else {
    submitManuscript({
      version,
      project,
      history,
      dispatch,
      redirectPath: submissionRedirect,
      values: pick(values, formSectionKeys),
    })
  }
}

export default compose(
  setDisplayName('SubmitWizard'),
  getContext({
    history: PropTypes.object,
    isFinal: PropTypes.bool,
    isFirst: PropTypes.bool,
    project: PropTypes.object,
    version: PropTypes.object,
    wizard: PropTypes.object,
    dispatchFns: PropTypes.object,
    confirmation: PropTypes.bool,
    toggleConfirmation: PropTypes.func,
  }),
  connect((state, { wizard: { formSectionKeys }, project, version }) => ({
    formValues: wizardSelector(state, ...formSectionKeys),
    isRevisionFlow: isRevisionFlow(state, project, version),
  })),
  withProps(
    ({
      version,
      isFirst,
      isFinal,
      isRevisionFlow,
      wizard: {
        formSectionKeys,
        backText = 'Back',
        nextText = 'Next',
        cancelText = 'Cancel',
        submitText = 'Submit Manuscript',
        revisionText = 'Submit Revision',
      },
    }) => ({
      readonly: !!get(version, 'submitted'),
      initialValues: pick(version, formSectionKeys),
      buttons: {
        backText: isFirst ? cancelText : backText,
        nextText: !isFinal // eslint-disable-line
          ? nextText
          : isRevisionFlow ? revisionText : submitText,
      },
    }),
  ),
  reduxForm({
    form: 'wizard',
    forceUnregisterOnUnmount: true,
    onChange: debounce(onChange, 1000, { maxWait: 5000 }),
    onSubmit,
  }),
)(WizardStep)
