const { omit, get, last } = require('lodash')

const config = require('config')

const statuses = config.get('statuses')

const publicStatusesPermissions = ['author', 'reviewer']

const parseAuthorsData = (coll, matchingCollPerm) => {
  if (['reviewer'].includes(matchingCollPerm.permission)) {
    coll.authors = coll.authors.map(a => omit(a, ['email']))
  }
}

const setPublicStatuses = (coll, matchingCollPerm) => {
  const status = get(coll, 'status') || 'draft'
  // coll.visibleStatus = statuses[status].public
  if (publicStatusesPermissions.includes(matchingCollPerm.permission)) {
    coll.visibleStatus = statuses[status].public
  }
}

const filterRefusedInvitations = (coll, user) => {
  const matchingInv = coll.invitations.find(inv => inv.userId === user.id)
  if (matchingInv === undefined) return null
  if (matchingInv.hasAnswer === true && !matchingInv.isAccepted) return null
  return coll
}

const filterObjectData = (
  collectionsPermissions = [],
  object = {},
  user = {},
) => {
  if (object.type === 'fragment') {
    const matchingCollPerm = collectionsPermissions.find(
      collPerm => object.id === collPerm.fragmentId,
    )
    if (matchingCollPerm === undefined) return null
    if (['reviewer'].includes(matchingCollPerm.permission)) {
      object.files = omit(object.files, ['coverLetter'])
      if (object.recommendations)
        object.recommendations = object.recommendations.filter(
          rec => rec.userId === user.id,
        )
    }
    parseAuthorsData(object, matchingCollPerm)
    if (['reviewer', 'handlingEditor'].includes(matchingCollPerm.permission)) {
      return filterRefusedInvitations(object, user)
    }
    return object
  }
  const matchingCollPerm = collectionsPermissions.find(
    collPerm => object.id === collPerm.id,
  )
  if (matchingCollPerm === undefined) return null
  setPublicStatuses(object, matchingCollPerm)

  return object
}

const getTeamsByPermissions = async (
  teamIds = [],
  permissions = [],
  TeamModel,
) =>
  (await Promise.all(
    teamIds.map(async teamId => {
      const team = await TeamModel.find(teamId)
      if (!permissions.includes(team.teamType.permissions)) {
        return null
      }
      return team
    }),
  )).filter(Boolean)

const heIsInvitedToFragment = async ({ user, Team, collectionId }) =>
  (await getTeamsByPermissions(user.teams, ['handlingEditor'], Team)).some(
    // user is a member of the team with access to the fragment's parent collection
    t => t.members.includes(user.id) && t.object.id === collectionId,
  )

const getUserPermissions = async ({
  user,
  Team,
  mapFn = t => ({
    objectId: t.object.id,
    objectType: t.object.type,
    role: t.teamType.permissions,
  }),
}) =>
  (await Promise.all(user.teams.map(teamId => Team.find(teamId)))).map(mapFn)

const isOwner = ({ user: { id }, object }) => {
  if (object.owners.includes(id)) return true
  return !!object.owners.find(own => own.id === id)
}

const hasPermissionForObject = async ({ user, object, Team, roles = [] }) => {
  const userPermissions = await getUserPermissions({
    user,
    Team,
  })

  return !!userPermissions.find(p => {
    const hasObject =
      p.objectId === get(object, 'fragment.id') ||
      p.objectId === get(object, 'fragment.collectionId')
    if (roles.length > 0) {
      return hasObject && roles.includes(p.role)
    }
    return hasObject
  })
}

const isHandlingEditor = ({ user, object }) =>
  get(object, 'collection.handlingEditor.id') === user.id

const isInDraft = fragment => !get(fragment, 'submitted')

const hasFragmentInDraft = async ({ object, Fragment }) => {
  const lastFragmentId = last(get(object, 'fragments'))
  const fragment = await Fragment.find(lastFragmentId)
  return isInDraft(fragment)
}

module.exports = {
  filterObjectData,
  parseAuthorsData,
  setPublicStatuses,
  getTeamsByPermissions,
  filterRefusedInvitations,
  isOwner,
  isHandlingEditor,
  getUserPermissions,
  heIsInvitedToFragment,
  hasPermissionForObject,
  isInDraft,
  hasFragmentInDraft,
}
